#ifndef __VIEWER__
#define __VIEWER__

#include <QGLViewer/qglviewer.h>
#include <QStringList>
#include <QColor>

namespace apig {
    class Renderable;
    class ProgGLSL;
    }

class Viewer : public QGLViewer {
    public:
        Viewer(QString fileName, QWidget* parent = NULL);
        ~Viewer();
    
    protected:
        virtual void init();
        virtual void draw();
        virtual void keyPressEvent(QKeyEvent *);
        
    private:
        apig::Renderable *object;
        apig::Renderable *ground;
        qglviewer::ManipulatedFrame *lightFrame;
        qglviewer::ManipulatedFrame *objectFrame;
        qglviewer::Camera *lightCamera;
        bool wireframe;
        int bgrIndex;
        static const QList<QColor> backgrounds;
        // for ShadowMap:
        void generateShadowFBO();
        int shadowMapWidth;
        int shadowMapHeight;
        // Hold the id of the framebuffer for light POV rendering
        GLuint fboId;
        // Z values will be rendered to this texture when using fboId framebuffer
        GLuint depthbuffer;
        // We keep a color texture for debugging purposes
        GLuint colorbuffer;
        bool debugBuffer;
        // shadowMap shader
        apig::ProgGLSL *shadowProg;
    };

#endif

