#ifndef __RENDERABLE__
#define __RENDERABLE__

#include "Vec3.h"

namespace apig {

    /** An interface for renderable objects. */
    class Renderable {
        public:
            virtual ~Renderable() {}
            virtual void init() {}      // this function is called once, when the OpenGL context is ready
            virtual void render() const = 0;
            virtual Vec3  center() const { return Vec3(0,0,0); }
            virtual float radius() const { return 1.0f; }
        };
    
    /** A cube renderer. */
    class RenderableCube : public Renderable {
        public:
            RenderableCube(float c) : c(c) {}
            
            virtual void render() const;
            
            //virtual Vec3  center() const;
            virtual float radius() const;
            
        private:
            float c;
        };
    
    /** A teapot renderer. */
    class RenderableTeapot : public Renderable {
        public:
            RenderableTeapot(float r) : r(r) {}
            
            virtual void render() const;
            
            //virtual Vec3  center() const;
            virtual float radius() const;
            
        private:
            float r;
        };
    
    }

#endif

