#ifndef __VIEWPORT_GL__
#define __VIEWPORT_GL__

#include <QDomDocument>
#include <QDomElement>

namespace apig {

    /** An interface to the OpenGL viewport.
     * It allows to easily set or get the current viewport and provides handful methods.
     */
    class ViewportGL {
        public:
            ViewportGL();   // prend la valeur du viewport du contexte OpenGL courant --> precondition : contexte OpenGL cree
            ViewportGL(int x, int y, int w, int h) : x(x), y(y), w(w), h(h) {}
            
            void getGL();           // actualise les parametres du viewport a partir de l'etat OpenGL
            void setGL() const;     // actualise l'etat OpenGL a partir des parametres du viewport
    
            ViewportGL subViewport(int wSub, int hSub);             // retourne le sous-viewport centre de taille (wSub, hSub)
            ViewportGL subViewport(int margin, float ratio);        // retourne le sous-viewport centre avec une marge 'margin' et le rapport wSub/hSub = ratio
            ViewportGL subViewport(int margin);                     // retourne le sous-viewport une marge 'margin' aux quatre bords
    
            ViewportGL subViewport(ViewportGL relativeView) const;
            ViewportGL relativeViewport(ViewportGL view) const;
            
            void setUnitSquare(ViewportGL subView) const;   // regle la PROJECTION matrix pour que le carre unite se mappe sur 'subView'.
            
            void pushScreenMatrices() const;
            void popScreenMatrices() const;
            
            void drawScreenQuad(bool filled = true, float z = 0) const;    // z en coordonnees ecran (dans [-1,1])
            
            // rapport hauteur / largeur du viewport :
            float ratio() const { return float(h) / float(w); }
            
            // ces methodes servent a passer dans un repere 2D orthonorme contenant [xmin,xmax]*[ymin,ymax] :
            static void pushFrame2D(float xmin, float xmax, float ymin, float ymax);
            static void popFrame2D();
            
            static void push(); // met le viewport courant sur la pile d'attributs OpenGL
            static void pop();  // enleve le viewport de la pile d'attributs OpenGL
            
            QDomElement domElement(const QString &name, QDomDocument &document) const;
            void initFromDOMElement(const QDomElement &element);
            
        public:
            int x, y, w, h;
        };
    
    }

#endif

